var LOGFILE_NAME = "user-defined-actions-config.log";
var l = new Logger(LOGFILE_NAME);
var lWarn = new Logger(LOGFILE_NAME);
var lInfo  = new Logger(LOGFILE_NAME);
var fLocalCopy = true;
// HLE ID >=1024

/**
*
*  UTF-8 data encode / decode
*  http://www.webtoolkit.info/
*
**/
 
var Utf8 = {
 
    // public method for url encoding
    encode : function (string) {
        string = string.replace(/\r\n/g,"\n");
        var utftext = "";
 
        for (var n = 0; n < string.length; n++) {
 
            var c = string.charCodeAt(n);
 
            if (c < 128) {
                utftext += String.fromCharCode(c);
            }
            else if((c > 127) && (c < 2048)) {
                utftext += String.fromCharCode((c >> 6) | 192);
                utftext += String.fromCharCode((c & 63) | 128);
            }
            else {
                utftext += String.fromCharCode((c >> 12) | 224);
                utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                utftext += String.fromCharCode((c & 63) | 128);
            }
 
        }
 
        return utftext;
    },
 
    // public method for url decoding
    decode : function (utftext) {
        var string = "";
        var i = 0;
        var c = c1 = c2 = 0;
 
        while ( i < utftext.length ) {
 
            c = utftext.charCodeAt(i);
 
            if (c < 128) {
                string += String.fromCharCode(c);
                i++;
            }
            else if((c > 191) && (c < 224)) {
                c2 = utftext.charCodeAt(i+1);
                string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
                i += 2;
            }
            else {
                c2 = utftext.charCodeAt(i+1);
                c3 = utftext.charCodeAt(i+2);
                string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
                i += 3;
            }
 
        }
 
        return string;
    }
 
};

/**
 * convert old entries if needed
 * @returns {undefined}
 */
function convertOldEntries()
{
    var version = '1.0.1';
    var versionNode = Property.getNode('version');
    if ((versionNode === null) || ( versionNode.getValue() < '1.0.1') ){
        //old version we need to convert to dsuid
        convertdSID2dSUID();
    }
    Property.setProperty('version' , version);
    Property.setFlag('version', 'ARCHIVE', true);
    Property.store();
}

/**
 * convert dsids to dsuid
 * @returns {undefined}
 */
function convertdSID2dSUID() {
    var subPropArray;
    var fMigrate=false;
    var tmpResult;
    var basePropertyArray = Property.getNode('/scripts/system-addon-user-defined-actions');
    if (basePropertyArray !== null) {
        subPropArray = basePropertyArray.getChildren();
        for (var iIndex = 0; iIndex < subPropArray.length; iIndex++) {
        	if (subPropArray[iIndex].getChild('actions')!=null) {
	            tmpResult = updateActions2dSUID(subPropArray[iIndex].getChild('actions'));
	            fMigrate = fMigrate || tmpResult;
        	}
        }
    }
    if (fMigrate) {
        Property.store();
    } else {
        lInfo.logln('updateDSID2DSUID - nothing to update');
    }    
}

/**
 * Converts all action entries in this subnode to new dsuid where appropriate
 * @param {Object} actions property tree subnode 'actions'
 * @returns {Boolean} true if at least one action was converted
 */
function updateActions2dSUID(actions) {
    try {
        var fMigrate=false;
        var dSID,dSUID;
        var subPropArray = actions.getChildren();
        lInfo.logln('Update action entries from dsid to dsuid');
        for (var iIndex = 0; iIndex < subPropArray.length; iIndex++) {
            switch (subPropArray[iIndex].getChild('type').getValue()) {
                case 'device-scene': //intent fallthrough
                case 'device-blink': //intent fallthrough
                case 'device-value':
                    {
	                	if (subPropArray[iIndex].getChild('dsid')!=null) {
	                        dSID = subPropArray[iIndex].getChild('dsid').getValue();
	                        dSUID = dSID2dSUID(dSID);
	                        Property.setProperty(subPropArray[iIndex].getPath()+'/dsuid',dSUID);
	                        Property.setFlag(subPropArray[iIndex].getPath()+'/dsuid','ARCHIVE',true);  
	                        subPropArray[iIndex].removeChild('dsid');
	                        fMigrate = true;
	                    }
                    }
                    break;
                case 'change-state': 
                    {
                        var actionName = subPropArray[iIndex].getChild('name').getValue();
                        var nameArray = actionName.split('.');
                        if ((nameArray[0] === 'dev') && (nameArray[1].length === 24)) {
                            var dsuid = dSID2dSUID(nameArray[1]);
                            actionName = '';
                            actionName = actionName.concat(nameArray[0],'.', dsuid, '.', nameArray[2]);
                            Property.setProperty(subPropArray[iIndex].getPath() + '/name', actionName);  
                            fMigrate=true;
                        }
                    }
                break;
                default: {lInfo.logln('--default nothing to do');}                
            }
        }
        return fMigrate;
    } catch (e) {
        lWarn.logln('Exception in backward-compatibility::updateActions2dSUID: ' + e);
        return fMigrate;
    }
}

function getNewIdValueFromBackup() {
    var idValue = Math.floor(new Date().getTime()/1000);
    var baseNode = Property.getNode('/scripts/system-addon-user-defined-actions');
    if (baseNode == null)
        return idValue;

    return idValue;
}

function restoreFromBackup() {
    //no type check done, because we suppose the backup to be consistent
    var baseNode = Property.getNode('/usr/');
    if (baseNode != null)
        Property.removeChild('events');
    var baseNodes = Property.getNode('/scripts/system-addon-user-defined-actions');
    if (baseNodes == null) {
        return;
    }

    var ffLocalCopy = fLocalCopy;
    fLocalCopy = false;
    for ( var iIndex = 0; iIndex < baseNodes.getChildren().length; iIndex++) {
        try {
            var oRootNode = baseNodes.getChildren()[iIndex];
            var esName = null;
            var idNode = oRootNode.getChild('id');
            if (idNode == null) {
                esName = getNewIdValueFromBackup();
            } else {
                esName = idNode.getValue();
            }
            
            if (oRootNode.getChild('name') != null) {
                storeSubParameter(esName + '/id', ""+esName);
                l.logln('restore node number ' + iIndex + " " + oRootNode.getChild('name'));
                storeSubParameter(esName + '/name', oRootNode.getChild('name').getValue());
                storeSubParameter(esName + '/disabled', oRootNode.getChild('disabled').getValue());
                storeSubParameter(esName + '/lastSaved', oRootNode.getChild('lastSaved').getValue());
                for ( var jIndex = 0; jIndex < oRootNode.getChild('actions').getChildren().length; jIndex++) {
                    l.logln('restore subnode: ' + jIndex);
                    var oSubobject = oRootNode.getChild('actions').getChildren()[jIndex];
                    storeSubParameter(esName + '/actions/' + jIndex + '/type', oSubobject.getChild('type').getValue());
                    switch (oSubobject.getChild('type').getValue()) {
	                    case 'zone-scene':
	                        storeSubParameter(esName + '/actions/' + jIndex + '/zone', oSubobject.getChild('zone').getValue());
	                        storeSubParameter(esName + '/actions/' + jIndex + '/group', oSubobject.getChild('group').getValue());
	                        storeSubParameter(esName + '/actions/' + jIndex + '/scene', oSubobject.getChild('scene').getValue());
	                        storeSubParameter(esName + '/actions/' + jIndex + '/force', oSubobject.getChild('force').getValue());
	                    break;
	                    case 'undo-zone-scene':
	                        storeSubParameter(esName + '/actions/' + jIndex + '/zone', oSubobject.getChild('zone').getValue());
	                        storeSubParameter(esName + '/actions/' + jIndex + '/group', oSubobject.getChild('group').getValue());
	                        storeSubParameter(esName + '/actions/' + jIndex + '/scene', oSubobject.getChild('scene').getValue());
	                        storeSubParameter(esName + '/actions/' + jIndex + '/force', oSubobject.getChild('force').getValue());
	                    break;
                        case 'device-scene':
                            storeSubParameter(esName + '/actions/' + jIndex + '/dsuid', oSubobject.getChild('dsuid').getValue());
                            storeSubParameter(esName + '/actions/' + jIndex + '/scene', oSubobject.getChild('scene').getValue());
                            storeSubParameter(esName + '/actions/' + jIndex + '/force', oSubobject.getChild('force').getValue());
                        break;
                        case 'device-value':
                            storeSubParameter(esName + '/actions/' + jIndex + '/value', oSubobject.getChild('value').getValue());
                            storeSubParameter(esName + '/actions/' + jIndex + '/dsuid', oSubobject.getChild('dsuid').getValue());
                        break;
                        case 'device-blink':
                            storeSubParameter(esName + '/actions/' + jIndex + '/dsuid', oSubobject.getChild('dsuid').getValue());
                        break;
                        case 'zone-blink':
                            storeSubParameter(esName + '/actions/' + jIndex + '/zone', oSubobject.getChild('zone').getValue());
                            storeSubParameter(esName + '/actions/' + jIndex + '/group', oSubobject.getChild('group').getValue());
                        break;
                        case 'custom-event':
                            storeSubParameter(esName + '/actions/' + jIndex + '/event', oSubobject.getChild('event').getValue());
                        break;
                        case 'url':
                            storeSubParameter(esName + '/actions/' + jIndex + '/url', oSubobject.getChild('url').getValue());
                        break;                        
                        case 'change-state':
                        	storeSubParameter(esName + '/actions/' + jIndex + "/statename" , oSubobject.getChild('statename').getValue());
                            storeSubParameter(esName + '/actions/' + jIndex + "/state" , oSubobject.getChild('state').getValue());
                        break;
                        case 'change-addon-state':
                        	storeSubParameter(esName + '/actions/' + jIndex + "/statename" , oSubobject.getChild('statename').getValue());
                            storeSubParameter(esName + '/actions/' + jIndex + "/addon-id" , oSubobject.getChild('addon-id').getValue());
                            storeSubParameter(esName + '/actions/' + jIndex + "/state" , oSubobject.getChild('state').getValue());
                        break;                                                
                    }
                    if (oSubobject.getChild('delay') !=null)
                        storeSubParameter(esName + '/actions/' + jIndex + '/delay', oSubobject.getChild('delay').getValue());
                }
            }
        } catch (e) {
            l.logln("Fehler beim RestoreFROMBackup " + iIndex + " : " + e);
            fLocalCopy = ffLocalCopy;
        }
    }
    for ( var iIndex = 0; iIndex < baseNodes.getChildren().length; iIndex++) {
        try {
            var oRootNode = baseNodes.getChildren()[iIndex];
            l.logln("Backcheck " + oRootNode.name + " value = " + isNaN(parseInt(oRootNode.name,10)));
            if (isNaN(parseInt(oRootNode.name))) {
                l.logln("remove " + oRootNode.getPath());
                baseNodes.removeChild(oRootNode);
            }
        } catch (e) {
            l.logln("Fehler beim Backcheck " + iIndex + " : " + e);
        }
    }
    Property.store();
    fLocalCopy = ffLocalCopy;
}

function getNewIdValue() {
    var idValue = Math.floor((new Date().getTime()) /1000);
    lInfo.logln('1 ID is ' + idValue);

    var baseNode = Property.getNode('/usr/events');
    if (baseNode == null)
        return idValue;

    return idValue;
}

function storeValue(sParameterString) {
    try {
        var success = true;
        var oParameterObject = JSON.parse(unescape(sParameterString));

        var idValue = null;
        lInfo.logln('ID is ' + oParameterObject.id);
        if (oParameterObject.id != null) {
            idValue = isNaN(parseInt("" + oParameterObject.id)) ? getNewIdValue() : parseInt("" + oParameterObject.id);  
        } else {
            idValue = getNewIdValue();
        }
        lInfo.logln('2 ID is ' + idValue);
        
        var esName = "" + idValue;

        var backupNode = Property.getNode('/scripts/system-addon-user-defined-actions');
        if (backupNode != null) {
            backupNode.removeChild(esName);
        }
        
        var baseNode = Property.getNode('/usr/events');
        if (baseNode != null) {
            baseNode.removeChild(esName);
        }
        
        //the unescape already ensures to be string
        storeSubParameter(esName + '/name', Utf8.encode(unescape(oParameterObject.name)));
        storeSubParameter(esName + '/id', "" + idValue);
        storeSubParameter(esName + '/disabled', false);
        var iValue = new Date().getTime();
        storeSubParameter(esName + '/lastSaved', "" + iValue);

        for ( var iIndex = 0; iIndex < oParameterObject.actions.length; iIndex++) {
            var oSubobject = oParameterObject.actions[iIndex];
            success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + '/type', oSubobject.type) ? success : false;
            switch (oSubobject.type) {
                case 'zone-scene':
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/zone', oSubobject.zone) ? success : false;
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/group', oSubobject.group) ? success : false;
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/scene', oSubobject.scene) ? success : false;
                    success = checkAndStoreBoolParameter(esName + '/actions/' + iIndex + '/force', oSubobject.force) ? success : false;
                break;
                case 'undo-zone-scene':
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/zone', oSubobject.zone) ? success : false;
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/group', oSubobject.group) ? success : false;
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/scene', oSubobject.scene) ? success : false;
                    success = checkAndStoreBoolParameter(esName + '/actions/' + iIndex + '/force', oSubobject.force) ? success : false;
                break;
                case 'device-scene':
                    success = checkAndStoreDSUIDParameter(esName + '/actions/' + iIndex + '/dsuid', oSubobject.dsuid, oSubobject.dsid) ? success : false;
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/scene', oSubobject.scene) ? success : false;
                    success = checkAndStoreBoolParameter(esName + '/actions/' + iIndex + '/force', oSubobject.force) ? success : false;
                break;
                case 'device-value':
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/value', oSubobject.value) ? success : false;
                    success = checkAndStoreDSUIDParameter(esName + '/actions/' + iIndex + '/dsuid', oSubobject.dsuid, oSubobject.dsid) ? success : false;
                break;
                case 'device-blink':
                    success = checkAndStoreDSUIDParameter(esName + '/actions/' + iIndex + '/dsuid', oSubobject.dsuid, oSubobject.dsid) ? success : false;
                break;
                case 'zone-blink':
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/zone', oSubobject.zone) ? success : false;
                    success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/group', oSubobject.group) ? success : false;
                break;
                case 'custom-event':
                    success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + '/event', oSubobject.event) ? success : false;
                break;
                case 'url':
                    oSubobject.url = oSubobject.url.replace(/&amp;/g, '&');
                    success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + '/url', Utf8.encode(unescape(oSubobject.url))) ? success : false;
                break;
                case 'change-state':
                	if (oSubobject.name!=null) {
                	    success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + "/statename" , oSubobject.name) ? success : false;
                	} else {
                	    success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + "/statename" , oSubobject.statename) ? success : false;
                	}
                	success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + "/state" , oSubobject.state) ? success : false;
                break;
                case 'change-addon-state':
                	if (oSubobject.name!=null) {
                	    success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + "/statename" , oSubobject.name) ? success : false;
                	} else {
                	    success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + "/statename" , oSubobject.statename) ? success : false;
                	}
                	success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + "/addon-id" , oSubobject.addonId) ? success : false;
                	success = checkAndStoreStringParameter(esName + '/actions/' + iIndex + "/state" , oSubobject.state) ? success : false;
                break;
            }
            if (oSubobject.delay!=null) {
                success = checkAndStoreIntParameter(esName + '/actions/' + iIndex + '/delay', oSubobject.delay) ? success : false;
            }
        }
        if (oParameterObject.metaData!=null) {
            success = checkAndStoreStringParameter(esName + '/metaData', oParameterObject.metaData) ? success : false;
        }
        
        if (!success) {
            //we could not store all data successfully so we cleanup
            baseNode.removeChild(esName);
            backupNode.removeChild(esName);
        }
        Property.store();
        return success;
    } catch (e) {
        l.logln("Fehler beim Speichern: " + e);
        for ( var i in e) {
            l.logln(i + ': ' + e[i]);
        }
        //we could not store all data successfully so we cleanup
        baseNode.removeChild(esName);
        backupNode.removeChild(esName);
        return false;
    }
}

l.logln("HLE-Config " + raisedEvent.name);
if (raisedEvent.name == 'system-addon-user-defined-actions-config') {
	Property.load();
    l.logln("HLE-Config param action " + raisedEvent.parameter.actions);
    if (raisedEvent.parameter.actions == 'save') {
        l.logln("HLE-Config param value " + raisedEvent.parameter.value);
        var succcess = storeValue(raisedEvent.parameter.value);
        Property.store();
        var evt;
        if (succcess) {
            evt = new Event("system-addon-user-defined-actions-saved", {complete:'success'});
        } else {
            evt = new Event("system-addon-user-defined-actions-saved", {complete:'failed'});
        }
        evt.raise();
    }
    if (raisedEvent.parameter.actions == 'enable') {
        l.logln("HLE-Config enable value " + raisedEvent.parameter.id);
        
        
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
            var sanityCheck = Property.getNode('/scripts/system-addon-user-defined-actions/' + oIDObject[sKey]);
            if (sanityCheck != null) {                
                storeSubParameter(oIDObject[sKey] + '/conditions/enabled', true);
                var iValue = new Date().getTime();        
                storeSubParameter(oIDObject[sKey] + '/lastSaved', "" + iValue);
            }
        }
        
        Property.store();
        var evt = new Event("system-addon-user-defined-actions-saved", {});
        evt.raise();
    }
    if (raisedEvent.parameter.actions == 'disable') {
        l.logln("HLE-Config disable value " + raisedEvent.parameter.id);

        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
            var sanityCheck = Property.getNode('/scripts/system-addon-user-defined-actions/' + oIDObject[sKey]);
            if (sanityCheck != null) {                
                storeSubParameter(oIDObject[sKey] + '/conditions/enabled', false);        
                var iValue = new Date().getTime();
                storeSubParameter(oIDObject[sKey] + '/lastSaved', "" + iValue);
            }        	
        }
        
        Property.store();
        var evt = new Event("system-addon-user-defined-actions-saved", {});
        evt.raise();
    }
    if (raisedEvent.parameter.actions == 'delete') {
        l.logln("HLE-Config delete value " + raisedEvent.parameter.id);

        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
            var baseNode = Property.getNode('/usr/events');
            if (baseNode != null)
                baseNode.removeChild(oIDObject[sKey]);
            baseNode = Property.getNode('/scripts/system-addon-user-defined-actions');
            if (baseNode != null)
                baseNode.removeChild(oIDObject[sKey]);        	
        }

        Property.store();
        var evt = new Event("system-addon-user-defined-actions-saved", {});
        evt.raise();
    }
    if (raisedEvent.parameter.actions == 'restore') {
        Property.load();
        l.logln('restore event');
        convertOldEntries();
        restoreFromBackup();
        var evt = new Event("system-addon-user-defined-actions-saved", {});
        evt.raise();
    } 
} else if (raisedEvent.name == 'running') {
    Property.load();
    l.logln('running event');
    convertOldEntries();
    restoreFromBackup();
}
